
#include "generalconfwidget.h"
#include "querywidget.h"

#include <KFileDialog>
#include <KMessageBox>
#include <KPushButton>
#include <KStandardDirs>

#include <QList>
#include <QVariant>

static const char* EXTENSION = ".xml";

class QueryConfigurationItem : public QListWidgetItem {
public:
    QueryConfigurationItem(const QueryConfiguration & config)
        : QListWidgetItem(),
          m_item(config), m_bCurrent(false)
    {
    }

    virtual QVariant data(int r) const {
        if( Qt::DisplayRole == r ) {
            return m_item.name();
        } else if ( Qt::FontRole == r ) {
            QFont f = QListWidgetItem::data(r).value<QFont>();
            if( m_bCurrent)
            {
                f.setBold(true);
                return f;
            } 
            else
            {
                return f;
            }
        }
        
        return QVariant();
    }

    inline const QueryConfiguration & configuration() const {
        return m_item;
    }

    inline void setConfiguration(const QueryConfiguration & conf) {
        m_item = conf;
    }

    inline void setCurrent(bool b) {
        m_bCurrent = b;
    }

    inline bool isCurrent() const {
        return m_bCurrent;
    }
private:
    QueryConfiguration m_item;
    bool m_bCurrent;

};

GeneralConfWidget::GeneralConfWidget(KConfigGroup & group, QWidget* parent) 
    : QWidget(parent), m_group(group)
{
    m_widget.setupUi(this);

    connect(m_widget.buttonNew, SIGNAL(clicked(bool)), this, SLOT(slotNewConf()));
    connect(m_widget.buttonEdit, SIGNAL(clicked(bool)), this, SLOT(slotEditConf()));
    connect(m_widget.buttonCurrent, SIGNAL(clicked(bool)), this, SLOT(slotSetCurrent()));
    connect(m_widget.buttonImport, SIGNAL(clicked(bool)), this, SLOT(slotImport()));
    connect(m_widget.buttonExport, SIGNAL(clicked(bool)), this, SLOT(slotExport()));
    connect(m_widget.buttonDownload, SIGNAL(clicked(bool)), this, SLOT(slotDownload()));
    connect(m_widget.listConfigurations, SIGNAL(itemSelectionChanged()), this, SLOT(slotEnableControls()));

    QPixmap newIcon = KIconLoader::global()->loadIcon( "document-new", KIconLoader::NoGroup, KIconLoader::SizeSmall );
    QPixmap editIcon = KIconLoader::global()->loadIcon( "edit-rename", KIconLoader::NoGroup, KIconLoader::SizeSmall );
    QPixmap currentIcon = KIconLoader::global()->loadIcon( "go-home", KIconLoader::NoGroup, KIconLoader::SizeSmall );
    QPixmap importIcon = KIconLoader::global()->loadIcon( "document-open", KIconLoader::NoGroup, KIconLoader::SizeSmall );
    QPixmap exportIcon = KIconLoader::global()->loadIcon( "document-save-as", KIconLoader::NoGroup, KIconLoader::SizeSmall );
    QPixmap downloadIcon = KIconLoader::global()->loadIcon( "get-hot-new-stuff", KIconLoader::NoGroup, KIconLoader::SizeSmall );
    m_widget.buttonNew->setIcon(newIcon);
    m_widget.buttonEdit->setIcon(editIcon);
    m_widget.buttonCurrent->setIcon(currentIcon);
    m_widget.buttonImport->setIcon(importIcon);
    m_widget.buttonExport->setIcon(exportIcon);
    m_widget.buttonDownload->setIcon(downloadIcon);


    loadConfigurations();
    slotEnableControls();
}

void GeneralConfWidget::loadConfigurations()
{
    QList<QString> filenames = m_group.readEntry("Filenames", QList<QString>());
    QString current = m_group.readEntry("Current", QString());

    QList<QueryConfiguration> configs;
    QList<QString>::const_iterator itFilenames = filenames.begin();
    while( itFilenames != filenames.end() )
    {
        QueryConfiguration config;
        config.readFromFile(*itFilenames);
        configs.append(config);

        ++itFilenames;
    }


    QList<QueryConfiguration>::const_iterator it = configs.begin();
    while( it != configs.end() ) 
    {
        QueryConfigurationItem* item = new QueryConfigurationItem( *it );
        QString expectedName = KStandardDirs::locateLocal("appdata", (*it).name()) + EXTENSION;
        item->setCurrent(expectedName == current);

        m_widget.listConfigurations->insertItem( -1, item );
        ++it;
    }
}

void GeneralConfWidget::slotNewConf()
{
    KSharedConfigPtr config = KGlobal::config();
    KConfigGroup group = config->group( QString("QueryDialog") );

    QueryConfiguration newConfig;
    newConfig.setName(i18n("New Configuration"));

    KDialog* dialog = new KDialog(this);
    dialog->restoreDialogSize(group);
    QueryWidget* queryWidget = new QueryWidget(newConfig, dialog->button(KDialog::Ok), dialog);
    dialog->setMainWidget(queryWidget);

    if(dialog->exec() == QDialog::Accepted) 
    {
        newConfig = queryWidget->configuration();
        QueryConfigurationItem* item = new QueryConfigurationItem( newConfig );
        m_widget.listConfigurations->insertItem( -1, item );
        dialog->saveDialogSize(group);
    }
}

void GeneralConfWidget::slotEditConf()
{
    KSharedConfigPtr config = KGlobal::config();
    KConfigGroup group = config->group( QString("QueryDialog") );

    QueryConfigurationItem* cur = static_cast<QueryConfigurationItem*>(m_widget.listConfigurations->currentItem());
    KDialog* dialog = new KDialog(this);
    dialog->restoreDialogSize(group);
    
    QueryWidget* queryWidget = new QueryWidget(cur->configuration(), dialog->button(KDialog::Ok), dialog);
    dialog->setMainWidget(queryWidget);

    if(dialog->exec() == QDialog::Accepted) 
    {
        cur->setConfiguration(queryWidget->configuration());
        dialog->saveDialogSize(group);
    }
}

void GeneralConfWidget::slotSetCurrent()
{
    for(int i=0; i<m_widget.listConfigurations->count(); i++) 
    {
        QueryConfigurationItem* item = static_cast<QueryConfigurationItem*>(m_widget.listConfigurations->item(i));
        item->setCurrent(false);
    }

    QueryConfigurationItem* cur = static_cast<QueryConfigurationItem*>(m_widget.listConfigurations->currentItem());
    cur->setCurrent(true);
}

void GeneralConfWidget::slotImport()
{
    QString filename = KFileDialog::getOpenFileName ( KUrl(), "*.xml", this, i18n("Import Configuration") );
    if( !filename.isEmpty() ) 
    {
        QueryConfiguration config;
        if(!config.readFromFile(filename))
        {
            KMessageBox::error(this, i18n("Cannot read from file %1.", filename));
        }
        
        for(int i=0; i<m_widget.listConfigurations->count(); i++) 
        {
            QueryConfigurationItem* item = static_cast<QueryConfigurationItem*>(m_widget.listConfigurations->item(i));
            if(item->configuration().name() == config.name())
            {
                KMessageBox::error(this, i18n("<p>Cannot import from file %1.<br>A configuration with this name doeas already exist.</p>", filename));
                return;
            }
        }

        QueryConfigurationItem* item = new QueryConfigurationItem( config );
        m_widget.listConfigurations->insertItem( -1, item );
    }
}

void GeneralConfWidget::slotExport()
{
    QString filename = KFileDialog::getSaveFileName ( KUrl(), "*.xml", this, i18n("Export Configuration") );
    if( !filename.isEmpty() ) 
    {
        QueryConfigurationItem* cur = static_cast<QueryConfigurationItem*>(m_widget.listConfigurations->currentItem());
        if(!cur->configuration().saveToFile(filename))
        {
            KMessageBox::error(this, i18n("Cannot save to file %1.", filename));
        }
    }
}

void GeneralConfWidget::slotEnableControls()
{
    m_widget.buttonEdit->setEnabled( m_widget.listConfigurations->currentItem() != NULL );
    m_widget.buttonCurrent->setEnabled( m_widget.listConfigurations->currentItem() != NULL );
    m_widget.buttonExport->setEnabled( m_widget.listConfigurations->currentItem() != NULL );
}

void GeneralConfWidget::slotDownload()
{
    KMessageBox::error(this, i18n("This feature has not yet been implemented."));
}

void GeneralConfWidget::slotSaveSettings() 
{    
    QList<QString> filenames;
    QString current;
    for(int i=0; i<m_widget.listConfigurations->count(); i++) 
    {
        QueryConfigurationItem* item = static_cast<QueryConfigurationItem*>(m_widget.listConfigurations->item(i));
        const QueryConfiguration & conf = item->configuration();

        QString filename = KStandardDirs::locateLocal("appdata", conf.name()) + EXTENSION;
        conf.saveToFile(filename);

        filenames.append(filename);

        if(item->isCurrent()) 
        {
            current = filename;
        }
    }

    m_group.writeEntry("Filenames", filenames);
    m_group.writeEntry("Current", current);
    m_group.sync();
}

#include "generalconfwidget.moc"

