#include "personalkwery.h"

#include <KLocale>
#include <KMessageBox>
#include <KConfigDialog>

#include <QAction>
#include <QGraphicsLinearLayout>
#include <QTextDocument>
#include <QPainter>
#include <QRectF>

#include "generalconfwidget.h"
#include "query.h"

#include <plasma/svg.h>
#include <plasma/theme.h>
#include <plasma/widgets/label.h>
#include <plasma/widgets/pushbutton.h>

static const char* DEFAULT_MESSAGE = "<p><h1>Welcome to <i>PersonalKwery</i></h1></p>"
    "<hr/>"
    "<p>PersonalKwery is a plasmoid which queries webservices in certain intervals "
    "and displays the results of the query on your desktop. The results can be formated "
    "using XSLT stylesheets.</p>"
    "<p><b>Please select a configuration in the preferences dialog.</b></p>";

PersonalKwery::PersonalKwery(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args),
      m_bIsError(false)
{
    m_query = new Query();
    m_document = new QTextDocument(this);
    m_refresh = new QAction(i18n("Refresh"), this);
    m_btnRefresh = new Plasma::PushButton(this);
    m_btnRefresh->setAction(m_refresh);
    m_btnRefresh->setVisible(false);
    m_lblError = new Plasma::Label(this);
    m_lblError->setVisible(false);

    QGraphicsLinearLayout* layout = new QGraphicsLinearLayout();
    QGraphicsLinearLayout* buttonLayout = new QGraphicsLinearLayout();
    buttonLayout->addStretch();
    buttonLayout->addItem(m_btnRefresh);
    buttonLayout->setStretchFactor(m_btnRefresh, 2);
    buttonLayout->addStretch();    
    layout->addStretch();
    layout->addItem(buttonLayout);
    layout->addItem(m_lblError);
    layout->addStretch();
    layout->setOrientation(Qt::Vertical);
    this->setLayout(layout);


    // this will get us the standard applet background, for free!
    setBackgroundHints(DefaultBackground);
    setHasConfigurationInterface(true);  
    setAspectRatioMode(Plasma::IgnoreAspectRatio);
    resize(400, 200); // Set default size

    m_text = DEFAULT_MESSAGE;
    m_document->setHtml(m_text);

    connect(m_query, SIGNAL(textChanged(const QString &)),
            this, SLOT(slotUpdateText(const QString &)));
    connect(m_query, SIGNAL(error(int, QString)),
            this, SLOT(slotUpdateError(int, QString)));

    connect(m_refresh, SIGNAL(triggered(bool)),
            this, SLOT(slotRefresh()));


    QPixmap refreshIcon = KIconLoader::global()->loadIcon( "view-refresh", KIconLoader::NoGroup, KIconLoader::SizeSmall );
    m_btnRefresh->setIcon(refreshIcon);
}


PersonalKwery::~PersonalKwery()
{
    delete m_query;

    if (hasFailedToLaunch()) {
        // Do some cleanup here
    } else {
        saveConfig();
    }
}

void PersonalKwery::init()
{
    loadConfig();
}

void PersonalKwery::enableControls() 
{
    m_btnRefresh->setVisible(m_bIsError);
    m_lblError->setVisible(m_bIsError);

    this->update();
}

void PersonalKwery::slotUpdateError(int errorCode, QString err)
{
    m_bIsError = true;

    QString msg = i18n("<qt><p>A network error occured: <b>%1<b><p><p>%2</p></qt>", errorCode, err);
    qDebug("%s", msg.toUtf8().data());

    m_lblError->setText(msg);
    this->enableControls();
}

void PersonalKwery::slotUpdateText(const QString & text)
{
    m_bIsError = false;

    m_text = text;
    m_document->setHtml(m_text);
    this->enableControls();
}

void PersonalKwery::paintInterface(QPainter* p,
        const QStyleOptionGraphicsItem*, const QRect& contentsRect)
{
    p->setRenderHint(QPainter::SmoothPixmapTransform);
    p->setRenderHint(QPainter::Antialiasing);

    if( !m_bIsError ) 
    {
        p->save();
        p->translate( contentsRect.x(), contentsRect.y() );
        m_document->setTextWidth( contentsRect.width() );
        m_document->drawContents(p, QRectF(0.0, 0.0, contentsRect.width(), contentsRect.height()));
        p->restore();
    }
}

void PersonalKwery::loadConfig()
{
    KConfigGroup group = this->config("PersonalKwerySize");
    this->restore(group); // restore geometry

    group = this->config("Configurations");
    QString current = group.readEntry("Current", QString());
    QueryConfiguration configuration;
    if( configuration.readFromFile(current) ) 
    {
        m_query->setConfig(configuration);
        m_query->start();
    }
    else
    {
        //KMessageBox::information(NULL, current);
        m_text = DEFAULT_MESSAGE;
        m_document->setHtml(m_text);
    }
}

void PersonalKwery::saveConfig()
{
    KConfigGroup group = this->config("PersonalKwerySize");
    this->save(group); // save geometry

    group.sync();
}

void PersonalKwery::slotRefresh()
{
    m_query->queryNowOnce();
}

void PersonalKwery::createConfigurationInterface(KConfigDialog* parent)
{
    KConfigGroup group = this->config("Configurations");
    GeneralConfWidget* general = new GeneralConfWidget(group);

    parent->addPage(general, i18n("General"), "configure" );
    parent->show();

    connect(parent, SIGNAL(okClicked()), general, SLOT(slotSaveSettings()));
    connect(parent, SIGNAL(applyClicked()), general, SLOT(slotSaveSettings()));
}

void PersonalKwery::configChanged()
{
    init();
    m_query->queryNowOnce();
}

#include "personalkwery.moc"
