
#include <QUrl>
#include <QNetworkReply>
#include <QTextBrowser>
#include <QTimer>

#include <KLocale>

#include <stdio.h>
#include <stdlib.h>

#include <string.h>
#include "query.h"

Query::Query() 
    : QObject()
{
    m_manager = new QNetworkAccessManager(this);
    connect(m_manager, SIGNAL(finished(QNetworkReply*)),
            this, SLOT(replyFinished(QNetworkReply*)));

    m_timer = new QTimer(this);
    connect(m_timer, SIGNAL(timeout()), this, SLOT(updateQuery()));
}

Query::~Query() 
{
}

void Query::start() 
{
    m_timer->stop(); // incase another timer is running

    m_queryUrl = this->getQueryUrl();
    updateQuery();
    m_timer->start(m_config.queryInterval());
}

void Query::queryNowOnce() 
{
    m_queryUrl = this->getQueryUrl();
    this->updateQuery();
}

void Query::updateQuery() 
{
    m_manager->get( QNetworkRequest(m_queryUrl) );
}

QUrl Query::getQueryUrl() const
{
    QUrl url(m_config.baseUrl());
    QMap<QString,QString>::const_iterator it = m_config.attributes().begin();

    while( it != m_config.attributes().end() ) 
    {
        url.addQueryItem( it.key(), it.value() );
        ++it;
    }

    return url;
}

void Query::replyFinished(QNetworkReply* reply)
{
    if( reply->error() != QNetworkReply::NoError ) 
    {
        emit error(reply->error(), i18n("Network error"));
        return;
    }

    reply->open(QIODevice::ReadOnly);
    QByteArray answer = reply->readAll();
    reply->close();

    m_lastWebServiceResult = QString::fromUtf8(answer.data());
    if(!m_config.xslt().isEmpty()) 
    {
        answer = xsltTransform(answer, m_config.xslt());
        m_lastTransformedResult = QString::fromUtf8(answer.data());
    }
    else
    {
        m_lastTransformedResult = m_lastWebServiceResult;
    }

    emit textChanged(QString::fromUtf8(answer.data()));
}

QByteArray Query::xsltTransform(const QByteArray& data, const QString & xslt) 
{
    QByteArray ret = data;
    xsltStylesheetPtr cur = NULL;
    xmlDocPtr doc = NULL, res = NULL, xsltDoc = NULL;
    int resultStringLen = 0;
    xmlChar* resultString = NULL;

    const char *params[2];
    params[0] = NULL;

    xmlSubstituteEntitiesDefault(1);
    xmlLoadExtDtdDefaultValue = 1;
    
    // TODO: clean up

    QByteArray xsltData = xslt.toUtf8();
    qDebug("Parsing xslt as xml..");
    xsltDoc = xmlParseMemory(xsltData.data(), xsltData.length());
    if( !xsltDoc ) 
    {
        qDebug("Unable to parse xslt as xml...");
        reportError(xmlGetLastError());
    }
    else
    {
        qDebug("Parsing xslt..");
        cur = xsltParseStylesheetDoc(xsltDoc);
        if( !cur ) 
        {
            qDebug("Unable to parse xslt...");
            reportError(xmlGetLastError());
        }
        else
        {
            qDebug("Parsing xml..");
            if( !m_config.returnedDataIsHtml() ) 
            {
                doc = xmlParseMemory(data.data(), data.length());
            }
            else
            {
                doc = htmlParseDoc((xmlChar*)data.data(), NULL);
            }

            if( !doc ) 
            {
                qDebug("Unable to parse xml...");
                reportError(xmlGetLastError());
            }
            else 
            {
                res = xsltApplyStylesheet(cur, doc, params);
                if( !res ) 
                {
                    qDebug("Unable to apply xslt...");
                    reportError(xmlGetLastError());
                }
                else
                {
                    xsltSaveResultToString(&resultString, &resultStringLen, res, cur);
                    ret = QByteArray((char*)resultString, resultStringLen);
                }
            }
        }
    }

    xsltFreeStylesheet(cur);
    xmlFreeDoc(res);
    xmlFreeDoc(doc);
    xmlFree(resultString);

    xsltCleanupGlobals();
    xmlCleanupParser();

    return ret;
}

void Query::reportError(xmlErrorPtr err) 
{
    if( err != NULL ) 
    {
        QString msg;

        msg.sprintf("%s in Line %i<br/>Extra1: %s<br/>Extra2: %s<br/>Extra3: %s<br/>",
                    err->message, err->line, err->str1, err->str2, err->str3 );

        emit error(0, msg);
    }
}


#include "moc_query.cpp"
