
#include "queryconfiguration.h"

#include <QFile>
#include <QDomDocument>
#include <QDomElement>
#include <QDomText>

static const char* ROOT_TAG_NAME = "queryconfiguration";
static const char* BASE_URL_TAG_NAME = "baseurl";
static const char* ATTRIBUTES_TAG_NAME  = "attributes";
static const char* ATTRIBUTE_TAG_NAME  = "attribute";
static const char* XSLT_TAG_NAME = "xslt";

static const char* VERSION_ATTRIBUTE_NAME = "version";
static const char* INTERVAL_ATTRIBUTE_NAME = "interval";
static const char* NAME_ATTRIBUTE_NAME = "name";
static const char* KEY_ATTRIBUTE_NAME = "key";
static const char* VALUE_ATTRIBUTE_NAME = "value";
static const char* DATAHTML_ATTRIBUTE_NAME = "dataIsHtml";

static const char* FILE_FORMAT_VERSION = "1.0";

QueryConfiguration::QueryConfiguration()
    : m_queryInterval( 3600 ), m_returnedDataIsHtml(false)
{
}

QueryConfiguration::QueryConfiguration(const QueryConfiguration & rhs)
{
    this->operator=(rhs);
}

const QueryConfiguration & QueryConfiguration::operator=(const QueryConfiguration & rhs )
{
    m_queryInterval = rhs.m_queryInterval;
    m_baseUrl = rhs.m_baseUrl;
    m_name = rhs.m_name;
    m_attributes = rhs.m_attributes;
    m_xslt = rhs.m_xslt;
    m_returnedDataIsHtml = rhs.m_returnedDataIsHtml;

    return *this;
}

bool QueryConfiguration::readFromFile(const QString & filename) 
{
    QDomDocument doc(ROOT_TAG_NAME);
    QFile file(filename);
    if (!file.open(QIODevice::ReadOnly))
        return false;

    if (!doc.setContent(&file)) {
        file.close();
        return false;
    }
    file.close();

    // print out the element names of all elements that are direct children
    // of the outermost element.
    QDomElement root = doc.documentElement();
    m_queryInterval = root.attribute(INTERVAL_ATTRIBUTE_NAME, QString::number(m_queryInterval)).toInt();
    m_name = root.attribute(NAME_ATTRIBUTE_NAME, m_name);
    m_returnedDataIsHtml = static_cast<bool>(root.attribute(DATAHTML_ATTRIBUTE_NAME, 
                                                            QString::number(static_cast<int>(m_returnedDataIsHtml))).toInt());

    QDomNode n = root.firstChild();
    while( !n.isNull() ) 
    {
        QDomElement e = n.toElement(); // try to convert the node to an element.
        if( !e.isNull() )
        {
            if( e.tagName() == BASE_URL_TAG_NAME )
            {
                m_baseUrl = e.text();
            }
            else if( e.tagName() == ATTRIBUTES_TAG_NAME )
            {
                QDomNodeList list = e.elementsByTagName(ATTRIBUTE_TAG_NAME);
                QMap<QString, QString> map;
                for(int i=0;i<list.count();i++)
                {
                    QDomElement attribute = list.at(i).toElement();
                    map.insert( attribute.attribute(KEY_ATTRIBUTE_NAME), attribute.attribute(VALUE_ATTRIBUTE_NAME) );
                }
                m_attributes = map;
            }
            else if( e.tagName() == XSLT_TAG_NAME ) 
            {
                m_xslt = e.text();
            }
        }

        n = n.nextSibling();
    }

    return true;
}

bool QueryConfiguration::saveToFile(const QString & filename) const
{
    QDomDocument doc(ROOT_TAG_NAME);
    QDomElement root = doc.createElement(ROOT_TAG_NAME);
    root.setAttribute(INTERVAL_ATTRIBUTE_NAME, m_queryInterval);
    root.setAttribute(NAME_ATTRIBUTE_NAME, m_name);
    root.setAttribute(DATAHTML_ATTRIBUTE_NAME, static_cast<int>(m_returnedDataIsHtml));
    root.setAttribute(VERSION_ATTRIBUTE_NAME, FILE_FORMAT_VERSION);
    doc.appendChild(root);

    QDomElement baseUrl = doc.createElement(BASE_URL_TAG_NAME);
    QDomText t1 = doc.createTextNode(m_baseUrl);
    baseUrl.appendChild(t1);
    root.appendChild(baseUrl);

    QDomElement attributes = doc.createElement(ATTRIBUTES_TAG_NAME);
    QMap<QString,QString>::const_iterator it = m_attributes.begin();

    while( it != m_attributes.end() ) 
    {
        QDomElement attribute = doc.createElement(ATTRIBUTE_TAG_NAME);
        attribute.setAttribute(KEY_ATTRIBUTE_NAME, it.key());
        attribute.setAttribute(VALUE_ATTRIBUTE_NAME, it.value());
        attributes.appendChild(attribute);
        ++it;
    }

    root.appendChild(attributes);


    QDomElement xslt = doc.createElement(XSLT_TAG_NAME);
    QDomText t2 = doc.createTextNode(m_xslt);
    xslt.appendChild(t2);
    root.appendChild(xslt);

    QFile file(filename);
    if (!file.open(QIODevice::WriteOnly))
        return false;

    file.write( doc.toByteArray() );
    file.close();

    return true;
}
