
#include "querywidget.h"

#include "ui_testwidget.h"

#include "query.h"
#include "queryconfiguration.h"

#include <KMessageBox>

QueryWidget::QueryWidget(const QueryConfiguration & configuration, QPushButton* buttonOk, QWidget* parent)
    : QWidget(parent), m_buttonOk(buttonOk)
{
    m_widget.setupUi(this);
    m_query = new Query();

    m_widget.name->setText(configuration.name());
    m_widget.baseUrl->setText(configuration.baseUrl());
    if(configuration.returnedDataIsHtml()) 
    {
        m_widget.resultHTML->setChecked(true);
    }
    else
    {
        m_widget.resultXML->setChecked(true);
    }
    m_widget.interval->setValue(configuration.queryInterval()/60/60);
    m_widget.xslt->setPlainText(configuration.xslt());

    QMap<QString,QString> attributes = configuration.attributes();
    m_widget.tableAttributes->setColumnCount(2);
    m_widget.tableAttributes->setRowCount(attributes.count());
    m_widget.tableAttributes->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_widget.tableAttributes->setSelectionMode(QAbstractItemView::SingleSelection);

    QMap<QString,QString>::const_iterator it = attributes.begin();
    int i = 0;
    while( it != attributes.end() ) 
    {
        m_widget.tableAttributes->setItem( i, 0, new QTableWidgetItem( it.key() ) );
        m_widget.tableAttributes->setItem( i, 1, new QTableWidgetItem( it.value() ) );
        ++it;
        ++i;
    }

    connect(m_widget.buttonAdd, SIGNAL(clicked(bool)), this, SLOT(slotAddAttribute()));
    connect(m_widget.buttonDelete, SIGNAL(clicked(bool)), this, SLOT(slotRemoveAttribute()));
    connect(m_widget.tableAttributes, SIGNAL(itemSelectionChanged()), this, SLOT(slotEnableControls()));
    connect(m_widget.name, SIGNAL(textChanged(const QString&)), this, SLOT(slotEnableControls()));
    connect(m_widget.baseUrl, SIGNAL(textChanged(const QString&)), this, SLOT(slotEnableControls()));
    connect(m_widget.buttonTest, SIGNAL(clicked(bool)), this, SLOT(slotTest()));
    connect(m_query, SIGNAL(textChanged(const QString&)), this, SLOT(slotTestQueryDone()));
    connect(m_query, SIGNAL(error(int, QString)), this, SLOT(slotTestQueryFailed(int, QString)));

    QPixmap addIcon = KIconLoader::global()->loadIcon( "list-add", KIconLoader::NoGroup, KIconLoader::SizeSmall );
    QPixmap deleteIcon = KIconLoader::global()->loadIcon( "list-remove", KIconLoader::NoGroup, KIconLoader::SizeSmall );
    QPixmap testIcon = KIconLoader::global()->loadIcon( "system-run", KIconLoader::NoGroup, KIconLoader::SizeSmall );
    m_widget.buttonAdd->setIcon(addIcon);
    m_widget.buttonDelete->setIcon(deleteIcon);
    m_widget.buttonTest->setIcon(testIcon);

    slotEnableControls();
}

QueryWidget::~QueryWidget()
{
    delete m_query;
}

void QueryWidget::slotAddAttribute()
{
    m_widget.tableAttributes->setRowCount(m_widget.tableAttributes->rowCount()+1);
}

void QueryWidget::slotRemoveAttribute()
{
    QModelIndexList list = m_widget.tableAttributes->selectionModel()->selectedRows();
    m_widget.tableAttributes->removeRow(list.first().row());
}

void QueryWidget::slotEnableControls()
{
    m_widget.buttonDelete->setEnabled(m_widget.tableAttributes->selectionModel()->hasSelection());

    m_buttonOk->setEnabled(
        !m_widget.name->text().isEmpty() &&
        !m_widget.baseUrl->text().isEmpty() );
}

QueryConfiguration QueryWidget::configuration() const
{
    QMap<QString, QString> attributes;
    for(int i=0;i<m_widget.tableAttributes->rowCount();i++)
    {
        attributes.insert( m_widget.tableAttributes->item(i,0)->text(),
                           m_widget.tableAttributes->item(i,1)->text());
    }

    QueryConfiguration config;
    config.setName( m_widget.name->text() );
    config.setBaseUrl( m_widget.baseUrl->text() );
    config.setReturnedDataIsHtml( m_widget.resultHTML->isChecked() );
    config.setXslt( m_widget.xslt->toPlainText() );
    config.setQueryInterval( m_widget.interval->value() * 60 * 60 );
    config.setAttributes( attributes );

    return config;
}

void QueryWidget::slotTest() 
{
    QueryConfiguration configuration = this->configuration();
    m_query->setConfig(configuration);
    m_query->queryNowOnce();
}

void QueryWidget::slotTestQueryDone()
{
    KSharedConfigPtr config = KGlobal::config();
    KConfigGroup group = config->group( QString("TestDialog") );
    KDialog* dialog = new KDialog(this);
    dialog->restoreDialogSize(group);
    QTabWidget* widget = new QTabWidget(dialog);
    Ui::TestWidget* testWidget = new Ui::TestWidget();
    testWidget->setupUi(widget);
    dialog->setMainWidget(widget);
    dialog->setButtons( KDialog::Close );

    testWidget->browser->setHtml(m_query->lastTransformedResult());
    testWidget->xslt->setPlainText(m_query->lastTransformedResult());
    testWidget->xml->setPlainText(m_query->lastWebServiceResult());
 
    dialog->exec();
    dialog->saveDialogSize(group);

    delete testWidget;
}

void QueryWidget::slotTestQueryFailed(int errorCode, QString err)
{
    QString msg = i18n("<qt><p>A network error occured: <b>%1<b><p><p>%2</p></qt>", errorCode, err);
    KMessageBox::error(this, msg);
}
